"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SymbolInstanceRenderer = void 0;
const SketchRenderer_1 = require("./SketchRenderer");
const models_1 = require("../jsonUtils/models");
const symbol_1 = require("../symbol");
const getImageDataFromURL_1 = require("../utils/getImageDataFromURL");
const findInGroup = (layer, type) => layer && layer.layers && layer.layers.find((l) => l._class === type);
const hasImageFill = (layer) => !!(layer.style && layer.style.fills && layer.style.fills.some((f) => f.image));
const removeDuplicateOverrides = (overrides) => {
    const seen = {};
    return overrides.filter(({ path }) => {
        const isDuplicate = typeof seen[path] !== 'undefined';
        seen[path] = true;
        return !isDuplicate;
    });
};
const extractOverridesReducer = (path) => (overrides, layer) => {
    if (layer._class === 'text') {
        return overrides.concat({
            type: 'stringValue',
            path: `${path}${layer.do_objectID}`,
            name: layer.name,
        });
    }
    if (layer._class === 'group') {
        // here we're doing some look-ahead to see if this group contains a group
        // that contains text. this is the structure that will appear if the user
        // creates a `<Text />` element with a custom name
        const subGroup = findInGroup(layer, 'group');
        const textLayer = findInGroup(subGroup, 'text');
        if (textLayer) {
            return overrides.concat({
                type: 'stringValue',
                path: `${path}${textLayer.do_objectID}`,
                name: textLayer.name,
            });
        }
        // here we're doing look-ahead to see if this group contains a shapeGroup
        // with an image fill. if it does we can do an image override on that
        // fill
        const shapeGroup = findInGroup(layer, 'shapeGroup');
        if (shapeGroup && hasImageFill(shapeGroup)) {
            return overrides.concat({
                type: 'image',
                path: `${path}${shapeGroup.do_objectID}`,
                name: layer.name,
            });
        }
    }
    if (layer._class === 'symbolInstance') {
        return overrides.concat({
            type: 'symbolID',
            path: `${path}${layer.do_objectID}`,
            name: layer.name,
            symbolID: layer.symbolID,
        });
    }
    if ((layer._class === 'shapeGroup' || layer._class === 'artboard' || layer._class === 'group') &&
        layer.layers) {
        return layer.layers.reduce(extractOverridesReducer(path), overrides);
    }
    return overrides;
};
const extractOverrides = (layers = [], path) => {
    const overrides = layers.reduce(extractOverridesReducer(path || ''), []);
    return removeDuplicateOverrides(overrides);
};
class SymbolInstanceRenderer extends SketchRenderer_1.SketchRenderer {
    renderGroupLayer({ layout, props, }) {
        const bridge = this.platformBridge;
        const masterTree = symbol_1.getSymbolMasterById(props.symbolID);
        if (!masterTree) {
            throw new Error('Trying to create a symbol instance for a Symbol Master that does not exists');
        }
        const symbolInstance = models_1.makeSymbolInstance(models_1.makeRect(layout.left, layout.top, layout.width, layout.height), masterTree.symbolID, props.name, props.resizingConstraint);
        const { overrides } = props;
        if (!overrides) {
            return symbolInstance;
        }
        const overridableLayers = extractOverrides(masterTree.layers);
        const overrideValues = overridableLayers.reduce(function inject(memo, reference) {
            if (reference.type === 'symbolID') {
                const newPath = `${reference.path}/`;
                const originalMaster = symbol_1.getSymbolMasterById(reference.symbolID);
                if (!originalMaster) {
                    return memo;
                }
                if (reference.name in overrides) {
                    const overrideValue = overrides[reference.name];
                    // @ts-ignore
                    const overrideSymbolId = overrideValue.symbolID;
                    if (typeof overrideValue !== 'function' || typeof overrideSymbolId !== 'string') {
                        throw new Error(`The overriden nested symbol needs to be the constructor of another symbol.\n\nIn Symbol Instance: "${props.name}"\nFor Override: "${reference.name}"`);
                    }
                    const replacementMaster = symbol_1.getSymbolMasterById(overrideSymbolId);
                    if (!replacementMaster) {
                        return memo;
                    }
                    if (originalMaster.frame.width !== replacementMaster.frame.width ||
                        originalMaster.frame.height !== replacementMaster.frame.height) {
                        throw new Error(`The overriden nested symbol needs to have the same dimensions.\n\nIn Symbol Instance: "${props.name}"\nFor Override: "${reference.name}"`);
                    }
                    memo.push(models_1.makeOverride(reference.path, reference.type, replacementMaster.symbolID));
                    extractOverrides(replacementMaster.layers, newPath).reduce(inject, memo);
                    return memo;
                }
                extractOverrides(originalMaster.layers, newPath).reduce(inject, memo);
                return memo;
            }
            if (!overrides.hasOwnProperty(reference.name)) {
                return memo;
            }
            const overrideValue = overrides[reference.name];
            if (reference.type === 'stringValue') {
                if (typeof overrideValue !== 'string') {
                    throw new Error(`The override value of a Text must be a string.\n\nIn Symbol Instance: "${props.name}"\nFor Override: "${reference.name}"`);
                }
                memo.push(models_1.makeOverride(reference.path, reference.type, overrideValue));
            }
            if (reference.type === 'image') {
                if (typeof overrideValue !== 'string') {
                    throw new Error(`The override value of an Image must be a url.\n\nIn Symbol Instance: "${props.name}"\nFor Override: "${reference.name}"`);
                }
                memo.push(models_1.makeOverride(reference.path, reference.type, models_1.makeJSONDataReference(getImageDataFromURL_1.getImageDataFromURL(bridge)(overrideValue))));
            }
            return memo;
        }, []);
        symbolInstance.overrideValues = overrideValues;
        return symbolInstance;
    }
}
exports.SymbolInstanceRenderer = SymbolInstanceRenderer;
